package excelv1

import (
	"bytes"
	"fmt"
	"github.com/tealeg/xlsx"
	"io"
	"net/http"
	"reflect"
	"time"
)

// ToExcel 生成io.ReadSeeker  参数 titleList 为Excel表头，dataList 为数据
func ToExcel(titleList []string, dataList []interface{}) (content io.ReadSeeker) {
	// 生成一个新的文件
	file := xlsx.NewFile()
	// 添加sheet页
	sheet, _ := file.AddSheet("Sheet1")
	// 插入表头
	titleRow := sheet.AddRow()
	for _, v := range titleList {
		cell := titleRow.AddCell()
		cell.Value = v
	}
	// 插入内容
	var types string = ""
	var innerTypes string = ""
	for _, v := range dataList {
		row := sheet.AddRow()
		vv := reflect.ValueOf(v)
		if innerTypes == "" {
			if vv.Kind() == reflect.Ptr {
				types = "Ptr"
				vv = vv.Elem()
			}
			if vv.Kind() == reflect.Struct {
				innerTypes = "Struct"
			} else if vv.Kind() == reflect.Slice {
				innerTypes = "Slice"
			}
		}
		if innerTypes == "Struct" {
			if types == "" {
				row.WriteStruct(&v, -1)
			} else {
				row.WriteStruct(v, -1)
			}

		} else if innerTypes == "Slice" {
			if types == "" {
				row.WriteSlice(&v, -1)
			} else {
				row.WriteSlice(v, -1)
			}
		}
	}

	var buffer bytes.Buffer
	_ = file.Write(&buffer)
	content = bytes.NewReader(buffer.Bytes())
	return
}

// ResponseXls 向前端返回Excel文件
// 参数 content 为上面生成的io.ReadSeeker， fileName 为返回前端的文件名
func ResponseXls(w http.ResponseWriter, req *http.Request, content io.ReadSeeker, fileName string) {
	w.Header().Add("Content-Disposition", fmt.Sprintf(`attachment; filename="%s"`, fileName))
	w.Header().Add("Content-Type", "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")
	http.ServeContent(w, req, fileName, time.Now(), content)
}

// ExportExcel 生成excel并导出
// 参数 fileName：文件名；titleList：表头；dataList：数据(数组的值只能是struct或者切片)
func ExportExcel(fileName string, titleList []string, dataList []interface{}, w http.ResponseWriter, req *http.Request) {
	content := ToExcel(titleList, dataList)
	ResponseXls(w, req, content, fileName)
}
